/**
 * Weather Source Forecast API
 * Daily and hourly forecast data for latitude/longitude points, ZIP/Postal Codes, Designated Market Areas, and OnPoint™ points. Hourly forecast is up to 10 days out from current day. Daily forecast is up to 15 days out from current day. The Weather Source Forecast API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */
package io.swagger.client.apis

import io.swagger.client.models.ErrorObj
import io.swagger.client.models.ForecastOnpointDayObj
import io.swagger.client.models.ForecastOnpointHourObj

import io.swagger.client.infrastructure.*

class OnpointsApi(basePath: kotlin.String = "https://forecast.weathersourceapis.com/v2") : ApiClient(basePath) {

    /**
     * A day of forecast data for an OnPoint™ point.
     * Returns a single day of forecast data for an OnPoint™ point. The provided date must exist within the current valid 15-day forecast range.
     * @param onpointId An OnPoint™ ID. 
     * @param date An date formatted as a RFC3339 date value. 
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastOnpointDayObj
     */
    @Suppress("UNCHECKED_CAST")
    fun onpointsOnpointIdDaysDateGet(onpointId: kotlin.Int, date: java.time.LocalDate, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastOnpointDayObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/onpoints/{onpointId}/days/{date}".replace("{" + "onpointId" + "}", "$onpointId").replace("{" + "date" + "}", "$date"), query = localVariableQuery
        )
        val response = request<ForecastOnpointDayObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastOnpointDayObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * A contiguous range of daily forecast data for an OnPoint™ point.
     * Returns a contiguous range of daily forecast data for an OnPoint™ point. The provided dates must exist within the current valid 15-day forecast range.
     * @param onpointId An OnPoint™ ID. 
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. 
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. 
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastOnpointDayObj
     */
    @Suppress("UNCHECKED_CAST")
    fun onpointsOnpointIdDaysDateStartdateEndGet(onpointId: kotlin.Int, dateStart: java.time.LocalDate, dateEnd: java.time.LocalDate, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastOnpointDayObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/onpoints/{onpointId}/days/{dateStart},{dateEnd}".replace("{" + "onpointId" + "}", "$onpointId").replace("{" + "dateStart" + "}", "$dateStart").replace("{" + "dateEnd" + "}", "$dateEnd"), query = localVariableQuery
        )
        val response = request<ForecastOnpointDayObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastOnpointDayObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * The complete daily forecast for an OnPoint™ point.
     * Returns the complete set of forecast data for an OnPoint™ point.&lt;br&gt;&lt;br&gt;**Alternatively using a query that specificies just the days required for your purposes will improve the query response time.**
     * @param onpointId An OnPoint™ ID. 
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastOnpointDayObj
     */
    @Suppress("UNCHECKED_CAST")
    fun onpointsOnpointIdDaysGet(onpointId: kotlin.Int, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastOnpointDayObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/onpoints/{onpointId}/days".replace("{" + "onpointId" + "}", "$onpointId"), query = localVariableQuery
        )
        val response = request<ForecastOnpointDayObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastOnpointDayObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * The complete hourly forecast for an OnPoint™ point.
     * Returns the complete set of forecast data for an OnPoint™ point.&lt;br&gt;&lt;br&gt;**Alternatively using a query that specificies just the hours required for your purposes will improve the query response time.**
     * @param onpointId An OnPoint™ ID. 
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(mslPres, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastOnpointHourObj
     */
    @Suppress("UNCHECKED_CAST")
    fun onpointsOnpointIdHoursGet(onpointId: kotlin.Int, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastOnpointHourObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/onpoints/{onpointId}/hours".replace("{" + "onpointId" + "}", "$onpointId"), query = localVariableQuery
        )
        val response = request<ForecastOnpointHourObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastOnpointHourObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * An hour of forecast data for an OnPoint™ point.
     * Returns a single hour of forecast data for an OnPoint™ point. The provided timestamp must exist within the current valid 10-day forecast range.
     * @param onpointId An OnPoint™ ID. 
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. 
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(mslPres, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastOnpointHourObj
     */
    @Suppress("UNCHECKED_CAST")
    fun onpointsOnpointIdHoursTimestampGet(onpointId: kotlin.Int, timestamp: java.time.LocalDateTime, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastOnpointHourObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/onpoints/{onpointId}/hours/{timestamp}".replace("{" + "onpointId" + "}", "$onpointId").replace("{" + "timestamp" + "}", "$timestamp"), query = localVariableQuery
        )
        val response = request<ForecastOnpointHourObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastOnpointHourObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * A contiguous range of hourly forecast data for an OnPoint™ point.
     * Returns a contiguous range of hourly forecast data for an OnPoint™ point. The provided timestamps must exist within the current valid 10-day forecast range.
     * @param onpointId An OnPoint™ ID. 
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. 
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. 
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(mslPres, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastOnpointHourObj
     */
    @Suppress("UNCHECKED_CAST")
    fun onpointsOnpointIdHoursTimestampStarttimestampEndGet(onpointId: kotlin.Int, timestampStart: java.time.LocalDateTime, timestampEnd: java.time.LocalDateTime, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastOnpointHourObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/onpoints/{onpointId}/hours/{timestampStart},{timestampEnd}".replace("{" + "onpointId" + "}", "$onpointId").replace("{" + "timestampStart" + "}", "$timestampStart").replace("{" + "timestampEnd" + "}", "$timestampEnd"), query = localVariableQuery
        )
        val response = request<ForecastOnpointHourObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastOnpointHourObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
}
