/**
 * Weather Source Forecast API
 * Daily and hourly forecast data for latitude/longitude points, ZIP/Postal Codes, Designated Market Areas, and OnPoint™ points. Hourly forecast is up to 10 days out from current day. Daily forecast is up to 15 days out from current day. The Weather Source Forecast API is built upon the [OnPoint™ Platform](https://weathersource.com/products/onpoint-platform/) which ensures data that is gap-free, homogeneous, and ready for immediate analysis. We offer the highest resolution grid on the market, covering every landmass in the world and up to 200 miles offshore.  An API key is required to authorize requests. If you do not have a Weather Source API key [sign up for a free 30-day developer account](https://developer.weathersourceapis.com/account/sign-up/). 
 *
 * OpenAPI spec version: 2.0.0
 * Contact: support@weathersource.com
 *
 * NOTE: This class is auto generated by the swagger code generator program.
 * https://github.com/swagger-api/swagger-codegen.git
 * Do not edit the class manually.
 */
package io.swagger.client.apis

import io.swagger.client.models.ErrorObj
import io.swagger.client.models.ForecastPointDayObj
import io.swagger.client.models.ForecastPointHourObj

import io.swagger.client.infrastructure.*

class PointsApi(basePath: kotlin.String = "https://forecast.weathersourceapis.com/v2") : ApiClient(basePath) {

    /**
     * A day of forecast data for a Latitude/Longitude point.
     * Returns a single day of forecast data for a Latitude/Longitude point. The provided date must exist within the current valid 15-day forecast range.
     * @param latitude A latitude value between -90° and 90°. 
     * @param longitude A longitude value between -180° (West) and 180° (East). 
     * @param date An date formatted as a RFC3339 date value. 
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastPointDayObj
     */
    @Suppress("UNCHECKED_CAST")
    fun pointsLatitudelongitudeDaysDateGet(latitude: kotlin.Float, longitude: kotlin.Float, date: java.time.LocalDate, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastPointDayObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/points/{latitude},{longitude}/days/{date}".replace("{" + "latitude" + "}", "$latitude").replace("{" + "longitude" + "}", "$longitude").replace("{" + "date" + "}", "$date"), query = localVariableQuery
        )
        val response = request<ForecastPointDayObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastPointDayObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * A contiguous range of daily forecast data for a Latitude/Longitude point.
     * Returns a contiguous range of daily forecast data for a Latitude/Longitude point. The provided dates must exist within the current valid 15-day forecast range.
     * @param latitude A latitude value between -90° and 90°. 
     * @param longitude A longitude value between -180° (West) and 180° (East). 
     * @param dateStart An date formatted as a RFC3339 date value that must occur temporally before dateEnd. 
     * @param dateEnd An date formatted as a RFC3339 date value that must occur temporally after dateStart. 
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastPointDayObj
     */
    @Suppress("UNCHECKED_CAST")
    fun pointsLatitudelongitudeDaysDateStartdateEndGet(latitude: kotlin.Float, longitude: kotlin.Float, dateStart: java.time.LocalDate, dateEnd: java.time.LocalDate, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastPointDayObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/points/{latitude},{longitude}/days/{dateStart},{dateEnd}".replace("{" + "latitude" + "}", "$latitude").replace("{" + "longitude" + "}", "$longitude").replace("{" + "dateStart" + "}", "$dateStart").replace("{" + "dateEnd" + "}", "$dateEnd"), query = localVariableQuery
        )
        val response = request<ForecastPointDayObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastPointDayObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * The complete daily forecast for a Latitude/Longitude point.
     * Returns the complete set of forecast data for a Latitude/Longitude point.&lt;br&gt;&lt;br&gt;**Alternatively using a query that specificies just the days required for your purposes will improve the query response time.**
     * @param latitude A latitude value between -90° and 90°. 
     * @param longitude A longitude value between -180° (West) and 180° (East). 
     * @param fields A comma separated list of field names to return. The date field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are supported: * all *(all fields)* * allCldCvr *(derivatives of cldCvr)* * allHum *(derivatives of relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(derivatives of mslPres, sfcPres)* * allRad *(derivatives of radSolar)* * allTemp *(derivatives of dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(derivatives of windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(derivatives of cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastPointDayObj
     */
    @Suppress("UNCHECKED_CAST")
    fun pointsLatitudelongitudeDaysGet(latitude: kotlin.Float, longitude: kotlin.Float, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastPointDayObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/points/{latitude},{longitude}/days".replace("{" + "latitude" + "}", "$latitude").replace("{" + "longitude" + "}", "$longitude"), query = localVariableQuery
        )
        val response = request<ForecastPointDayObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastPointDayObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * The complete hourly forecast for a Latitude/Longitude point.
     * Returns the complete set of forecast data for a Latitude/Longitude point.&lt;br&gt;&lt;br&gt;**Alternatively using a query that specificies just the hours required for your purposes will improve the query response time.**
     * @param latitude A latitude value between -90° and 90°. 
     * @param longitude A longitude value between -180° (West) and 180° (East). 
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(mslPres, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastPointHourObj
     */
    @Suppress("UNCHECKED_CAST")
    fun pointsLatitudelongitudeHoursGet(latitude: kotlin.Float, longitude: kotlin.Float, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastPointHourObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/points/{latitude},{longitude}/hours".replace("{" + "latitude" + "}", "$latitude").replace("{" + "longitude" + "}", "$longitude"), query = localVariableQuery
        )
        val response = request<ForecastPointHourObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastPointHourObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * An hour of forecast data for a Latitude/Longitude point.
     * Returns a single hour of forecast data for a Latitude/Longitude point. The provided timestamp must exist within the current valid 10-day forecast range.
     * @param latitude A latitude value between -90° and 90°. 
     * @param longitude A longitude value between -180° (West) and 180° (East). 
     * @param timestamp A timestamp formatted as an RFC3339 date-time value. The UTC offset is required. 
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(mslPres, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastPointHourObj
     */
    @Suppress("UNCHECKED_CAST")
    fun pointsLatitudelongitudeHoursTimestampGet(latitude: kotlin.Float, longitude: kotlin.Float, timestamp: java.time.LocalDateTime, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastPointHourObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/points/{latitude},{longitude}/hours/{timestamp}".replace("{" + "latitude" + "}", "$latitude").replace("{" + "longitude" + "}", "$longitude").replace("{" + "timestamp" + "}", "$timestamp"), query = localVariableQuery
        )
        val response = request<ForecastPointHourObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastPointHourObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
    /**
     * A contiguous range of hourly forecast data for a Latitude/Longitude point.
     * Returns a contiguous range of hourly forecast data for a Latitude/Longitude point. The provided timestamps must exist within the current valid 10-day forecast range.
     * @param latitude A latitude value between -90° and 90°. 
     * @param longitude A longitude value between -180° (West) and 180° (East). 
     * @param timestampStart A timestamp formatted as an RFC3339 date-time value that must occur temporally before timestampEnd. The UTC offset is required. 
     * @param timestampEnd A timestamp formatted as an RFC3339 date-time value that must occur temporally after timestampStart. The UTC offset is required. 
     * @param fields A comma separated list of field names to return. The timestamp field is always returned.  **Limiting the query to needed fields will improve the query response time.**  In addition to individual field names, following convenience field groups are also supported: * all *(all fields)* * allCldCvr *(cldCvr)* * allHum *(relHum, spcHum)* * allPrecip *(precip, precipProb, snowfall, snowfallProb)* * allPres *(mslPres, sfcPres)* * allRad *(radSolar)* * allTemp *(dewPt, feelsLike, heatIndex, temp, wetBulb, windChill)* * allWind *(windDir, windDir80m, windDir100m, windSpd, windSpd80m, windSpd100m)* * popular *(cldCvr, feelsLike, precip, relHum, snowfall, temp, windDir, windSpd)*  
     * @param unitScale The unit scale for returned values.&lt;br&gt; (optional, default to IMPERIAL)
     * @return ForecastPointHourObj
     */
    @Suppress("UNCHECKED_CAST")
    fun pointsLatitudelongitudeHoursTimestampStarttimestampEndGet(latitude: kotlin.Float, longitude: kotlin.Float, timestampStart: java.time.LocalDateTime, timestampEnd: java.time.LocalDateTime, fields: kotlin.Array<kotlin.String>, unitScale: kotlin.String? = null): ForecastPointHourObj {
        val localVariableQuery: MultiValueMap = mapOf("fields" to toMultiValue(fields.toList(), "csv"), "unitScale" to listOf("$unitScale"))
        val localVariableConfig = RequestConfig(
                RequestMethod.GET,
                "/points/{latitude},{longitude}/hours/{timestampStart},{timestampEnd}".replace("{" + "latitude" + "}", "$latitude").replace("{" + "longitude" + "}", "$longitude").replace("{" + "timestampStart" + "}", "$timestampStart").replace("{" + "timestampEnd" + "}", "$timestampEnd"), query = localVariableQuery
        )
        val response = request<ForecastPointHourObj>(
                localVariableConfig
        )

        return when (response.responseType) {
            ResponseType.Success -> (response as Success<*>).data as ForecastPointHourObj
            ResponseType.Informational -> TODO()
            ResponseType.Redirection -> TODO()
            ResponseType.ClientError -> throw ClientException((response as ClientError<*>).body as? String ?: "Client error")
            ResponseType.ServerError -> throw ServerException((response as ServerError<*>).message ?: "Server error")
        }
    }
}
